#    Copyright (C) 2014 Assaf Gordon <assafgordon@gmail.com>
#
#    This file is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This file is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this file. If not, see <http://www.gnu.org/licenses/>.
#
#    Written by Assaf Gordon

import re
from subprocess import Popen, PIPE
import os
import pprint
import json
from redis import Redis

tarball_url_re = re.compile ('^(http|ftp|https)://[A-Za-z0-9\-]+\.[A-Za-z0-9\.]+/[A-Za-z0-9\/\-\_\.]+(\.tar\.(gz|bz2|xz)|\.git)$')
git_url_re = re.compile('^git://[A-Za-z0-9\-]+\.[A-Za-z0-9\.]+/[A-Za-z0-9\/\-\_\.]+$');

redis_connection=Redis()

def get_current_active_job_num():
    num = redis_connection.get("gnu_eval_current_active_job")
    try:
        num = int(num)
    except:
        num = 0
    return num

def set_current_active_job_num(num):
    try:
        num = int(num)
    except:
        num = 0
    redis_connection.set("gnu_eval_current_active_job", num);

def get_current_submitted_job_num():
    num = redis_connection.get("gnu_eval_current_submitted_job")
    try:
        num = int(num)
    except:
        num = 0
    return num

def allocate_new_submitted_job_num():
    num = redis_connection.incr("gnu_eval_current_submitted_job");
    try:
        num = int(num)
    except:
        num = 0
    return num

def acceptable_source_url(url):
    if tarball_url_re.match(url) and git_url_re.match(url):
        return True
    return False

def add_complete_job(idnum,project_name,source_url):
    try:
        job_data = [ str(idnum), str(project_name), str(source_url) ]
        job_data_json = json.dumps(job_data)
        redis_connection.lpush("gnu-eval-completed-jobs", job_data_json)
    except:
        pass

def get_completed_jobs():
    jobs = []
    try:
        jobs_json = redis_connection.lrange("gnu-eval-completed-jobs",0,100)
        for j in jobs_json:
            job = json.loads(j)
            jobs.append(job)
    except:
        pass
    return jobs

def sanitize_project_name(name):
    return re.sub(r"[^A-Za-z0-9\.\-\_ ]","",name);

def do_gnu_eval(idnum,output_dir,project_name,source_url):
    try:
        idnum = int(idnum)
    except ValueError:
        return "error: invalid idnum (%s) parameter" % (idnum)
    if not project_name:
        project_name = "(unamed project)"
    if not output_dir:
        return "error: output directory is empty"
    if not os.path.isdir(output_dir):
        return "error: output directory (%s) is not a valid directory" % (output_dir)
    if acceptable_source_url(source_url):
        return "error: Source URL (%s) does not match allowed pattern" % (source_url)

    project_name = sanitize_project_name(str(project_name))
    source_url = str(source_url)
    idnum = str(idnum)

    eval_script = "../gsv-eval-remote.sh"
    args = [ eval_script, output_dir + str(idnum) + ".html", project_name, source_url ]

    child = Popen(args, shell=False, stdout=PIPE, stderr=PIPE)

    output = child.communicate();
    exitcode = child.returncode

    if exitcode == 0:
        set_current_active_job_num (idnum)
        add_complete_job (idnum, project_name, source_url)
        return "ok";
    else:
        return output[1] # stderr
